from datetime import datetime
from django.conf import settings
from django.contrib.auth.models import User
from django.utils.timezone import get_current_timezone
from importlib import import_module
from taggit.models import Tag

def view_filter(**kwargs):
    """
    Filters blog posts by year/month/day, category, tag or author
    """

    from bambu_blog.models import Post, Category
    posts = Post.objects.select_related().live()

    if 'year' in kwargs:
        posts = posts.filter(
            date__year = int(kwargs['year'])
        )

        if 'month' in kwargs:
            posts = posts.filter(
                date__month = int(kwargs['month'])
            )

            if 'day' in kwargs:
                posts = posts.filter(
                    date__day = int(kwargs['day'])
                )

    if 'category' in kwargs:
        posts = posts.filter(categories__slug = kwargs['category'])
    elif 'tag' in kwargs:
        posts = posts.filter(tags__slug = kwargs['tag'])
    elif 'username' in kwargs:
        posts = posts.filter(author__username = kwargs['username'])

    return posts

def title_parts(**kwargs):
    """
    Returns a tuple of strings that can be used to build the ``<title>`` tag for an HTML
    template
    """
    from bambu_blog.models import Category

    title_parts = [u'Blog']
    if 'year' in kwargs:
        if 'month' in kwargs:
            if 'day' in kwargs:
                title_parts.insert(0,
                    datetime(
                        int(kwargs['year']),
                        int(kwargs['month']),
                        int(kwargs['day'])
                    ).replace(tzinfo = get_current_timezone()).strftime('%B %d, %Y')
                )
            else:
                title_parts.insert(0,
                    datetime(
                        int(kwargs['year']),
                        int(kwargs['month']),
                        1
                    ).replace(tzinfo = get_current_timezone()).strftime('%B %Y')
                )
        else:
            title_parts.insert(0, kwargs['year'])

    if 'category' in kwargs:
        category = Category.objects.get(slug = kwargs['category'])
        title_parts.insert(0, category.name)
    elif 'tag' in kwargs:
        tag = Tag.objects.get(slug = kwargs['tag'])
        title_parts.insert(0, tag.name)
    elif 'username' in kwargs:
        author = User.objects.get(username = kwargs['username'])
        title_parts.insert(0, author.get_full_name() or author.username)

    return title_parts

def get_post_image(post):
    """
    Gets the first attachment matching common 'image' MIME types
    """

    image_types = (
        'image/bmp', 'image/x-windows-bmp', 'image/gif',
        'image/jpeg', 'image/pjpeg', 'image/png', 'image/tiff'
    )

    images = post.attachments.filter(mimetype__in = image_types)[:1]
    if images.exists():
        try:
            url = images[0].file.url
            if url.startswith('/'):
                url = settings.MEDIA_URL[:-1] + url

            return url
        except:
            pass

    return ''

def get_comments_form():
    """
    Gets the class of the form used to post comments to a blog post
    (``bambu_comments.forms.CommentForm`` by default)
    """

    klass = getattr(settings, 'BLOG_COMMENTS_FORM',
        'bambu_comments.forms.CommentForm'
    )

    module, dot, klass = klass.rpartition('.')
    module = import_module(module)
    klass = getattr(module, klass)

    return klass



from django.conf import settings
from pyquery import PyQuery
from django.template.defaultfilters import truncatewords
from html2text import html2text
import string

__version__ = '3.2.2'
default_app_config = 'bambu_blog.apps.BlogConfig'
EXCERPT_LENGTH = getattr(settings, 'BLOG_EXCERPT_LENGTH', 30)

def plainify(html):
    doc = PyQuery('<body>%s</body>' % html)
    doc('img, audio, video, iframe, embed, object, script').remove()

    for a in doc('a, i, b, strong, em'):
        PyQuery(a).replaceWith(
            PyQuery(a).html()
        )

    for b in doc('blockquote'):
        PyQuery(b).replaceWith(
            PyQuery(b).html()
        )

    for a in doc('h1, h2, h3, h4, h5, h6'):
        PyQuery(a).replaceWith('<p>%s:</p>' % PyQuery(a).text())

    for p in doc('p'):
        t = (PyQuery(p).text() or '').strip()

        if not t:
            PyQuery(p).remove()
            continue

        if not t[-1] in string.punctuation:
            t += '. '

        if t.startswith('http:') or t.startswith('https:'):
            PyQuery(p).remove()

        if t.startswith('[') and t.endswith(']'):
            PyQuery(p).remove()

        PyQuery(p).html(t)

    for li in doc('li'):
        t = (PyQuery(li).text() or '').strip()
        if not t:
            PyQuery(li).remove()
            continue

        if not t[-1] in string.punctuation:
            t += '.'

        PyQuery(li).html(t)

    return html2text(
        doc.html()
    )

def excerpt(html, length = None):
    for tag in ('h1', 'h2', 'h3', 'h4', 'h5', 'h6'):
        h = html.find('<%s' % tag)
        if h > -1:
            html = html[:h]

    doc = PyQuery('<body>%s</body>' % html)
    doc('img, audio, video, iframe, embed, object, script').remove()

    for a in doc('a, i, b, strong, em'):
        PyQuery(a).replaceWith(
            PyQuery(a).html()
        )

    for b in doc('blockquote'):
        PyQuery(b).replaceWith(
            '"%s"' % PyQuery(b).html()
        )

    for p in doc('p'):
        t = (PyQuery(p).text() or '').strip()

        if not t:
            PyQuery(p).remove()
            continue

        if not t[-1] in string.punctuation:
            t += '. '

        if t.startswith('http:') or t.startswith('https:'):
            PyQuery(p).remove()

        if t.startswith('[') and t.endswith(']'):
            PyQuery(p).remove()

        PyQuery(p).html(t)

    for li in doc('li'):
        t = (PyQuery(li).text() or '').strip()
        if not t:
            PyQuery(li).remove()
            continue

        if not t[-1] in string.punctuation:
            t += '.'

        PyQuery(li).html(t)

    excerpt = ''
    for p in doc('p'):
        if excerpt:
            excerpt += ' '

        excerpt += html2text(PyQuery(p).html())
        if len(excerpt.split()) >= 30:
            break

    if not length:
        length = EXCERPT_LENGTH

    return truncatewords(excerpt, length)



from django.template import Library
from django.db.models import Count
from django.core.exceptions import FieldError
from taggit import VERSION as TAGGIT_VERSION
from taggit.managers import TaggableManager
from taggit.models import TaggedItem, Tag

T_MAX = 6.0
T_MIN = 2.0

register = Library()

def get_weight_fun(t_min, t_max, f_min, f_max):
    def weight_fun(f_i, t_min = t_min, t_max = t_max, f_min = f_min, f_max = f_max):
        if f_max == f_min:
            mult_fac = 1.0
        else:
            mult_fac = float(t_max - t_min) / float(f_max - f_min)
            
        return int(t_max - (f_max - f_i) * mult_fac)
    return weight_fun

@register.inclusion_tag('blog/tag-cloud.inc.html')
def tag_cloud():
    try:
        queryset = Tag.objects.annotate(
            num_times = Count('taggeditem_items')
        )
    except FieldError:
        queryset = Tag.objects.annotate(
            num_times = Count('taggit_taggeditem_items')
        )
    
    num_times = queryset.values_list('num_times', flat = True)
    if(len(num_times) == 0):
        return {}
    
    weight_fun = get_weight_fun(T_MIN, T_MAX, min(num_times), max(num_times))
    queryset = queryset.order_by('name')
    
    for tag in queryset:
        tag.weight = weight_fun(tag.num_times)
    
    return {
        'tags': queryset
    }


from bambu_attachments.models import Attachment
from bambu_blog.models import Post, Category
from bambu_comments.models import Comment
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site
from django.core.files import File
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from django.template.defaultfilters import slugify
from django.utils.timezone import get_current_timezone
from optparse import make_option
from os import sys, path, write, close, remove

try:
    from xml.etree import ElementTree
except ImportError:
    from elementtree import ElementTree

from datetime import datetime
from lxml import etree
from pyquery import PyQuery
from tempfile import mkstemp
from urlparse import urlparse
import requests

XML_NS = {
    'wp': 'http://wordpress.org/export/1.2/',
    'content': 'http://purl.org/rss/1.0/modules/content/',
    'dc': 'http://purl.org/dc/elements/1.1/'
}

class Command(BaseCommand):
    help = 'Import blog posts from a WordPress XML file'
    args = '<filename>'

    def handle(self, *args, **options):
        xml = ElementTree.parse(open(args[0], 'r'))
        channel = xml.find('channel')

        def node_text(node, namespace = None, parent = None):
            if namespace:
                item = (parent or channel).find(ns(namespace, node))
            else:
                item = (parent or channel).find(node)

            if not item is None:
                return item.text

            return None

        def ns(n, o):
            return '{%s}%s' % (XML_NS[n], o)

        if channel is None:
            raise CommandError('Cannot find <channel> tag')

        title = node_text('title')
        if title:
            print(u'Blog title: %s' % title)

        link = node_text('link')
        if link:
            print(u'Blog URL: %s' % link)

        description = node_text('description')
        if description:
            print(u'Blog description: %s' % description)

        mappings = {
            'users': {},
            'posts': {},
            'categories': {},
            'comments': {}
        }

        content_type = ContentType.objects.get_for_model(Post)
        site = get_current_site()
        postmeta = {}

        print
        with transaction.commit_manually():
            try:
                for author in channel.findall(ns('wp', 'wp_author')):
                    username = node_text('author_login', 'wp', author)
                    email = node_text('author_email', 'wp', author)
                    display_name = node_text('author_display_name', 'wp', author)
                    user = None

                    if not username:
                        continue

                    if display_name:
                        display_name = '%s (%s)' % (username, display_name)
                    else:
                        display_name = username

                    try:
                        user = User.objects.get(username__iexact = username)
                    except User.DoesNotExist:
                        if email:
                            try:
                                user = User.objects.get(email__iexact = email)
                            except:
                                pass

                    if not user:
                        new_username = raw_input('Map old user %s to a user in your database: ' % display_name)
                        if not new_username:
                            continue

                        while True:
                            try:
                                user = User.objects.get(username__iexact = new_username)
                                break
                            except User.DoesNotExist:
                                new_username = raw_input('User not found. Please try again ,or press Enter to ignore: ')
                                if not new_username:
                                    print 'Ignoring user %s' % username
                                    break

                    if user:
                        mappings['users'][username] = user
                        print 'Mapping user %s to %s' % (
                            username, user.get_full_name() or user.username
                        )

                for item in channel.findall('item'):
                    id = node_text('post_id', 'wp', item)
                    title = node_text('title', parent = item)
                    url = node_text('link', parent = item)
                    kind = node_text('post_type', 'wp', item)
                    parent = node_text('post_parent', 'wp', item)
                    published = node_text('status', 'wp', item) == 'publish'
                    author = node_text('creator', 'dc', item)
                    date = node_text('post_date_gmt', 'wp', item)
                    body = node_text('encoded', 'content', item) or u''

                    try:
                        id = int(id)
                    except ValueError:
                        continue

                    if not date:
                        continue

                    try:
                        date = datetime.strptime(date,
                            '%Y-%m-%d %H:%M:%S'
                        ).replace(
                            tzinfo = get_current_timezone()
                        )
                    except:
                        continue

                    try:
                        parent = int(parent)
                    except ValueError:
                        continue

                    if parent:
                        continue

                    if not author:
                        continue

                    if not mappings['users'].has_key(author):
                        continue

                    author = mappings['users'][author]
                    if not kind in ('post', 'page'):
                        continue

                    if kind == 'post':
                        try:
                            post = Post.objects.get(title = title, date = date)
                            print 'Updating %s "%s"' % (kind, title)
                        except Post.DoesNotExist:
                            post = Post(
                                title = title,
                                slug = title and slugify(title) or None,
                                date = date,
                                published = published,
                                broadcast = True,
                                author = author
                            )

                            print 'Creating %s "%s"' % (kind, title)
                    else:
                        continue

                    post.body = body
                    post.save()
                    mappings['posts'][id] = post

                    for category in item.findall('category'):
                        domain = category.get('domain')
                        slug = category.get('nicename')

                        if not category.text:
                            continue

                        if domain == 'category':
                            if not mappings['categories'].has_key(slug):
                                mappings['categories'][slug], created = Category.objects.get_or_create(
                                    name = category.text,
                                    slug = slugify(category.text)
                                )

                                if created:
                                    print '- Created category "%s"' % category.text

                            post.categories.add(
                                mappings['categories'][slug]
                            )
                        elif domain == 'post_tag':
                            if category.text.startswith('"') and category.text.endswith('"'):
                                post.tags.add(category.text[1:-1])
                            else:
                                post.tags.add(category.text)

                    for comment in item.findall(ns('wp', 'comment')):
                        comment_id = node_text('comment_id', 'wp', comment)
                        comment_name = node_text('comment_author', 'wp', comment)
                        comment_email = node_text('comment_author_email', 'wp', comment)
                        comment_url = node_text('comment_author_url', 'wp', comment)
                        comment_date = node_text('comment_date_gmt', 'wp', comment)
                        comment_type = node_text('comment_type', 'wp', comment)
                        comment_body = node_text('comment_content', 'wp', comment)
                        comment_parent = node_text('comment_parent', 'wp', comment)
                        comment_approved = node_text('comment_approved', 'wp', comment) == '1'

                        try:
                            comment_id = int(comment_id)
                        except ValueError:
                            continue

                        try:
                            comment_parent = int(comment_parent)
                        except ValueError:
                            comment_parent = 0

                        try:
                            comment_date = datetime.strptime(
                                comment_date, '%Y-%m-%d %H:%M:%S'
                            ).replace(
                                tzinfo = get_current_timezone()
                            )
                        except:
                            continue

                        if not comment_name:
                            continue

                        if not comment_type or comment_type == 'comment':
                            try:
                                comment = post.comments.get(
                                    name = comment_name,
                                    sent = comment_date
                                )
                            except Comment.DoesNotExist:
                                comment = Comment(
                                    name = comment_name,
                                    website = comment_url,
                                    email = comment_email or '',
                                    sent = comment_date,
                                    approved = comment_approved,
                                    body = comment_body,
                                    content_type = content_type,
                                    object_id = post.pk
                                )

                                print '- Comment by %s' % comment_name

                            comment.save(notify = False)
                            mappings['comments'][comment_id] = comment

                    postmeta[id] = {}
                    for meta in item.findall(ns('wp', 'postmeta')):
                        meta_key = node_text('meta_key', 'wp', meta)
                        meta_value = node_text('meta_value', 'wp', meta)
                        postmeta[id][meta_key] = meta_value

                    ai = 1
                    for subitem in channel.findall('item'):
                        subid = node_text('post_id', 'wp', subitem)
                        subparent_id = node_text('post_parent', 'wp', subitem)
                        subtitle = node_text('title', parent = subitem)
                        suburl = node_text('link', parent = subitem)
                        subkind = node_text('post_type', 'wp', subitem)
                        suburl = node_text('attachment_url', 'wp', subitem)

                        try:
                            subparent_id = int(subparent_id)
                        except ValueError:
                            continue

                        if not suburl:
                            continue

                        if subkind != 'attachment' or subparent_id != id:
                            continue

                        s, d, p, a, q, f = urlparse(suburl)
                        d, s, filename = p.rpartition('/')

                        try:
                            attachment = post.attachments.get(
                                title = subtitle or filename
                            )
                        except Attachment.DoesNotExist:
                            print '- Downloading %s' % filename

                            response = requests.get(suburl)
                            handle, tmp = mkstemp(
                                path.splitext(filename)[-1]
                            )

                            write(handle, response.content)
                            close(handle)

                            attachment = Attachment(
                                title = subtitle or filename,
                                file = File(open(tmp, 'r'), name = filename),
                                content_type = content_type,
                                object_id = post.pk
                            )

                            if '_thumbnail_id' in postmeta[id]:
                                if unicode(postmeta[id]['_thumbnail_id']) == unicode(subid):
                                    attachment.featured = True

                            attachment.save()
                            remove(tmp)

                        if post.body:
                            html = PyQuery('<body>' + post.body + '</body>')
                            for a in html(
                                'a[href="%(url)s"], [src="%(url)s"]' % {
                                    'url': suburl
                                }
                            ):
                                a = PyQuery(a)
                                a.replaceWith('\n\n[attachment %d]\n\n' % ai)

                            post.body = html.html()

                        ai += 1

                    if post.body:
                        html = PyQuery('<body>' + post.body + '</body>')
                        for a in html('a[href]'):
                            href = a.get('href')
                            if href.startswith(link):
                                href = href.replace(link, 'http://%s' % site.domain)

                            a = PyQuery(a)

                        for p in html('p'):
                            p = PyQuery(p)
                            p.replaceWith('\n\n%s\n\n' % p.html())

                        html('.alignright').addClass('pull-right').removeClass('alignright')
                        html('.alignleft').addClass('pull-left').removeClass('alignleft')

                        while '\n\n\n' in post.body:
                            post.body = post.body.replace('\n\n\n', '\n\n')

                        while '\r\r\r' in post.body:
                            post.body = post.body.replace('\r\r\r', '\r\r')

                        post.body = html.html()
                        post.body = post.body.replace('<br />', '  \n')
                        post.body = post.body.replace('<br/>', '  \n')
                        post.body = post.body.replace('<br>', '  \n')

                        while post.body.startswith('\n'):
                            post.body = post.body[1:]

                        while post.body.endswith('\n'):
                            post.body = post.body[:-1]

                        while post.body.startswith('\r'):
                            post.body = post.body[1:]

                        while post.body.endswith('\r'):
                            post.body = post.body[:-1]

                        while post.body.startswith('\t'):
                            post.body = post.body[1:]

                        post.body = post.body.strip()

                    post.save()

                transaction.commit()
            except:
                transaction.rollback()
                raise



from haystack import indexes
from bambu_blog.models import Post
from datetime import datetime

class PostIndex(indexes.SearchIndex, indexes.Indexable):
    text = indexes.CharField(document = True, use_template = True)
    author = indexes.CharField(model_attr = 'author')
    date = indexes.DateTimeField(model_attr = 'date')
    
    def get_model(self):
        return Post
    
    def index_queryset(self, using = None):
        return super(PostIndex, self).index_queryset().live()


from django.contrib.sitemaps import Sitemap
from bambu_blog.models import Post

class BlogSitemap(Sitemap):
    changefreq = 'daily'
    priority = 0.5
    
    def items(self):
        return Post.objects.live()
    
    def lastmod(self, obj):
        return obj.date


# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import taggit.managers
import bambu_attachments.helpers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, db_index=True)),
                ('slug', models.SlugField(unique=True, max_length=100)),
            ],
            options={
                'ordering': ('name',),
                'db_table': 'blog_category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=100, null=True, blank=True)),
                ('slug', models.SlugField(max_length=100)),
                ('date', models.DateTimeField(db_index=True)),
                ('published', models.BooleanField(default=True)),
                ('broadcast', models.BooleanField(default=False, editable=False)),
                ('body', models.TextField()),
                ('excerpt', models.TextField(null=True, editable=False, blank=True)),
                ('css', models.TextField(null=True, blank=True)),
                ('author', models.ForeignKey(related_name='blog_posts', to=settings.AUTH_USER_MODEL)),
                ('categories', models.ManyToManyField(related_name='posts', to='bambu_blog.Category', blank=True)),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
            ],
            options={
                'ordering': ('-date',),
                'db_table': 'blog_post',
                'get_latest_by': 'date',
            },
        ),
        migrations.CreateModel(
            name='PostUpload',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('file', models.FileField(max_length=255, upload_to=bambu_attachments.helpers.upload_attachment_file)),
                ('url', models.CharField(max_length=255, db_index=True)),
                ('size', models.PositiveIntegerField(editable=False)),
                ('mimetype', models.CharField(max_length=50, editable=False, db_index=True)),
            ],
            options={
                'db_table': 'blog_post_upload',
            },
        ),
    ]



from bambu_blog.helpers import view_filter, title_parts
from bambu_blog.models import Category
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.contrib.syndication.views import Feed
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.template import Template, Context
from django.utils.feedgenerator import Rss201rev2Feed

POSTS_PER_PAGE = getattr(settings, 'BLOG_POSTS_PER_PAGE', 10)

class ContentFeed(Rss201rev2Feed):
    def root_attributes(self):
        attrs = super(ContentFeed, self).root_attributes()
        attrs['xmlns:content'] = 'http://purl.org/rss/1.0/modules/content/'
        return attrs

    def add_item_elements(self, handler, item):
        super(ContentFeed, self).add_item_elements(handler, item)

        handler.startElement('content:encoded', {})
        handler._write('<![CDATA[%s]]>' % item['content_encoded'])
        handler.endElement('content:encoded')

class BlogFeed(Feed):
    feed_type = ContentFeed

    def items(self, obj):
        posts = view_filter(**obj).live()
        paginator = Paginator(posts, POSTS_PER_PAGE)

        try:
            posts = paginator.page(self.page)
        except PageNotAnInteger:
            posts = paginator.page(1)
        except EmptyPage:
            posts = paginator.page(paginator.num_pages)

        return posts.object_list

    def get_object(self, request, **kwargs):
        self.page = request.GET.get('page')
        return kwargs

    def link(self):
        return 'http://%s/' % get_current_site().domain

    def title(self, obj):
        site = get_current_site()
        parts = title_parts(**obj)
        parts.append(site.name)
        parts.reverse()

        return ': '.join(parts)

    def categories(self):
        return Category.objects.filter(
            posts__isnull = False
        ).distinct().values_list('name', flat = True)

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.excerpt

    def item_enclosure_url(self, item):
        enclosures = item.attachments.filter(
            mimetype__in = (
                'video/avi',
                'video/msvideo',
                'video/x-msvideo',
                'audio/mpeg3',
                'audio/x-mpeg-3',
                'video/mp4',
                'video/x-m4v',
                'video/mpeg'
            )
        )

        if enclosures.count() == 1:
            enclosure = enclosures[0]
            url = enclosure.file.url
            q = url.find('?')
            if q > -1:
                url = url[:q]

            return url.replace('https://', 'http://')

    def item_enclosure_length(self, item):
        enclosures = item.attachments.filter(
            mimetype__in = (
                'video/avi',
                'video/msvideo',
                'video/x-msvideo',
                'audio/mpeg3',
                'audio/x-mpeg-3',
                'video/x-m4v',
                'video/mp4',
                'video/mpeg'
            )
        )

        if enclosures.count() == 1:
            return enclosures[0].size

    def item_enclosure_mime_type(self, item):
        enclosures = item.attachments.filter(
            mimetype__in = (
                'video/avi',
                'video/msvideo',
                'video/x-msvideo',
                'audio/mpeg3',
                'audio/x-mpeg-3',
                'audio/x-wav',
                'video/x-m4v',
                'video/mp4',
                'video/mpeg'
            )
        )

        if enclosures.count() == 1:
            return enclosures[0].mimetype

    def item_pubdate(self, item):
        return item.date

    def item_author_name(self, item):
        return item.author.get_full_name() or item.author.username

    def item_author_email(self, item):
        return item.author.email

    def item_categories(self, item):
        return item.categories.values_list('name', flat = True)

    # def item_copyright(self, item):
    #         return item.channel.copyright

    # def item_subtitle(self, item):
    #         return item.subtitle

    def item_extra_kwargs(self, item):
        kwargs = super(BlogFeed, self).item_extra_kwargs(item)
        kwargs['content_encoded'] = ' '.join(
            Template(
                '{% load markup oembed attachments %}' \
                '{{ post.body|markdown|oembed|attachments:post }}'
            ).render(
                Context(
                    {
                        'post': item,
                    }
                )
            ).splitlines()
        )

        return kwargs



from django.contrib import admin
from django.contrib.admin.helpers import AdminForm, InlineAdminFormSet, AdminErrorList
from django.conf import settings
from django.conf.urls import patterns
from django.utils.timezone import now
from django.utils.translation import ugettext as _
from bambu_blog.models import *
from bambu_blog.forms import PostForm
from bambu_attachments.admin import AttachmentInline

class PostAdmin(admin.ModelAdmin):
    list_display = ('title', 'date', 'published')
    list_filter = ('published', 'categories')
    date_hierarchy = 'date'
    prepopulated_fields = {
        'slug': ('title',)
    }
    
    search_fields = ('title', 'body')
    form = PostForm
    
    fieldsets = (
        (
            u'Basics',
            {
                'fields': ('title', 'date', 'published')
            }
        ),
        (
            u'Post content',
            {
                'fields': ('body',)
            }
        ),
        (
            u'Metadata',
            {
                'fields': ('slug', 'author', 'tags', 'categories'),
                'classes': ('collapse', 'closed')
            }
        ),
        (
            u'Advanced',
            {
                'fields': ('css',)
            }
        )
    )

    inlines = (AttachmentInline,)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == 'author':
            kwargs['initial'] = request.user

        return super(PostAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
        
    class Media:
        js = 'grappelli' in settings.INSTALLED_APPS and (
            'grappelli/tinymce/jscripts/tiny_mce/tiny_mce.js',
            'js/tinymce_setup.js'
        ) or ()

admin.site.register(Post, PostAdmin)

class CategoryAdmin(admin.ModelAdmin):
    list_display = ('name',)
    prepopulated_fields = {
        'slug': ('name',)
    }

admin.site.register(Category, CategoryAdmin)



from django.test import TestCase
from django.contrib.auth.models import User
from django.utils.timezone import now
from taggit.models import Tag
from bambu_blog.models import Category, Post

class BlogModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username = 'user',
            password = 'blink',
            email = 'user@example.com'
        )
    
    def tearDown(self):
        self.user.delete()
    
    def test_categories(self):
        category = Category.objects.create(
            name = 'Category name',
            slug = 'category-name'
        )
        
        category.name = 'Category renamed'
        category.save()
        category.delete()
    
    def test_untitled_post(self):
        post = Post.objects.create(
            author = self.user,
            body = 'Something',
            date = now()
        )
        
        post.body = 'Something else'
        post.save()
        post.delete()
    
    def test_titled_post(self):
        post = Post.objects.create(
            title = 'Something',
            slug = 'something',
            author = self.user,
            date = now()
        )
        
        post.title = 'Something else'
        post.save()
        post.delete()
    
    def test_categorised_post(self):
        category = Category.objects.create(
            name = 'Category name',
            slug = 'category-name'
        )
        
        post = Post.objects.create(
            title = 'Something',
            slug = 'something',
            author = self.user,
            date = now()
        )
        
        post.categories.add(category)
        self.assertEqual(Post.objects.filter(categories__slug = 'category-name').count(), 1)
        
        post.categories.remove(category)
        self.assertEqual(Post.objects.filter(categories__slug = 'category-name').count(), 0)
        post.delete()
        category.delete()
    
    def test_tagged_post(self):
        post = Post.objects.create(
            title = 'Something',
            slug = 'something',
            author = self.user,
            date = now()
        )
        
        post.tags.add('tag')
        self.assertEqual(Post.objects.filter(tags__slug = 'tag').count(), 1)
        
        post.tags.remove('tag')
        self.assertEqual(Post.objects.filter(tags__slug = 'tag').count(), 0)
        
        post.delete()
        Tag.objects.get(slug = 'tag').delete()


from bambu_cron import CronJob, MINUTE, site
from bambu_blog.models import Post
from django.utils.timezone import now

class PostJob(CronJob):
    frequency = MINUTE
    
    def run(self, logger):
        date = now()
        
        for post in Post.objects.filter(date__lte = date, broadcast = False):
            post.publish()
            post.save()

site.register(PostJob)


from django.db import models
from django.utils.timezone import now

class PostQuerySet(models.query.QuerySet):
    """
    A custom queryset that adds a few utility functions to the standard one
    """

    def live(self):
        """Returns only posts in the future that are marked as published"""
        return self.filter(
            date__lte = now(),
            published = True
        )

    def css(self, rendered = False):
        """Returns rendered CSS for all of the posts in the query"""
        if rendered:
            return '\n\n'.join(
                [
                    (post.render_css() or u'') for post in self.all()
                ]
            )
        else:
            return '\n\n'.join(
                [
                    (css or u'') for css in self.values_list('css', flat = True)
                ]
            )

    def with_featured_attachments(self, only = False):
        queryset = self.extra(
            select = {
                'featured_attachment_file': 'SELECT file FROM attachments_attachment AS a ' \
                'INNER JOIN django_content_type AS t ON t.id = a.content_type_id ' \
                'WHERE t.app_label = \'bambu_blog\' AND t.model = \'post\' ' \
                'AND a.object_id = blog_post.id AND a.featured'
            }
        )

        if only:
            queryset = queryset.filter(
                attachments__featured = True
            ).distinct('date', 'pk')

        return queryset

    def with_comment_counts(self):
        return self.extra(
            select = {
                'comment_count': 'SELECT COUNT(*) FROM comments_comment AS c ' \
                'INNER JOIN django_content_type AS t ON t.id = c.content_type_id ' \
                'WHERE t.app_label = \'bambu_blog\' AND t.model = \'post\' ' \
                'AND c.object_id = blog_post.id AND c.approved'
            }
        )



from django.conf.urls import patterns, include, url
from bambu_blog.views import posts, post, post_comment
from bambu_blog.feeds import BlogFeed
from django.conf import settings

if 'bambu_bootstrap' in settings.INSTALLED_APPS:
    from bambu_bootstrap.decorators import body_classes
else:
    def body_classes(view, *classes):
        return view

urlpatterns = patterns('',
    url(r'^$', body_classes(posts, 'blog'), name = 'blog_posts'),
    url(r'^feed/$', BlogFeed(), name = 'blog_posts_feed'),
    url(r'^(?P<year>\d{4})/$', body_classes(posts, 'blog', 'blog-year'), name = 'blog_posts_by_year'),
    url(r'^(?P<year>\d{4})/feed/$', BlogFeed(), name = 'blog_posts_by_year_feed'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/$', body_classes(posts, 'blog', 'blog-month'), name = 'blog_posts_by_month'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/feed/$', BlogFeed(), name = 'blog_posts_by_month_feed'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/$', body_classes(posts, 'blog', 'blog-day'), name = 'blog_posts_by_day'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/feed/$', BlogFeed(), name = 'blog_posts_by_day_feed'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>[\w-]+)/$', body_classes(post, 'blog', 'blog-post'), name = 'blog_post'),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})/(?P<slug>[\w-]+)/comment/$', body_classes(post_comment, 'blog', 'blog-post', 'comment-post'), name = 'blog_post_comment'),
    url(r'^tag/(?P<tag>[\w-]+)/$', body_classes(posts, 'blog'), name = 'blog_posts_by_tag'),
    url(r'^tag/(?P<tag>[\w-]+)/feed/$', BlogFeed(), name = 'blog_posts_by_tag_feed'),
    url(r'^category/(?P<category>[\w-]+)/$', body_classes(posts, 'blog'), name = 'blog_posts_by_category'),
    url(r'^category/(?P<category>[\w-]+)/feed/$', BlogFeed(), name = 'blog_posts_by_category_feed'),
    url(r'^author/(?P<username>[\w]+)/$', body_classes(posts, 'blog'), name = 'blog_posts_by_author'),
    url(r'^author/(?P<username>[\w]+)/feed/$', BlogFeed(), name = 'blog_posts_by_author_feed')
)



from bambu_navigation import MenuBuilder, site

class BlogMenuBuilder(MenuBuilder):
    def register_partials(self):
        return (
            {
                'name': 'blog',
                'description': 'Adds a link to the blog'
            },
        )
    
    def add_to_menu(self, name, items, **kwargs):
        items.append(
            {
                'url': ('blog_posts',),
                'title': u'Blog',
                'selection': 'blog',
                'order': 100
            }
        )
        
site.register(BlogMenuBuilder)


from django import forms
from django.conf import settings
from django.utils.timezone import now
from bambu_blog.models import Post

class PostForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(PostForm, self).__init__(*args, **kwargs)
        self.fields['date'].initial = now()
        self.fields['tags'].required = False

        if 'markitup' in settings.INSTALLED_APPS:
            from markitup.widgets import MarkItUpWidget
            self.fields['body'].widget = MarkItUpWidget()
        elif 'grappelli' in settings.INSTALLED_APPS:
            classes = self.fields['css'].widget.attrs.get('class', '').split(' ')
            classes.append('mceNoEditor')
            self.fields['css'].widget.attrs['class'] = ' '.join(set(sorted(classes)))

        self.fields['css'].label = u'Custom CSS'

    class Meta:
        model = Post
        fields = (
            'author',
            'title',
            'slug',
            'date',
            'published',
            'body',
            'css',
            'tags',
            'categories'
        )



from bambu_blog.models import Post, Category, PostUpload
from bambu_xmlrpc import handler, XMLRPCException
from django.conf import settings
from django.contrib.auth import authenticate
from django.contrib.sites.shortcuts import get_current_site
from django.core.files import File
from django.core.urlresolvers import reverse
from django.db import transaction
from django.template.defaultfilters import slugify
from django.utils.timezone import now
from logging import getLogger
from os import path, close, write
from pyquery import PyQuery
from tempfile import mkstemp

STAFF_ONLY = getattr(settings, 'BLOG_XMLRPC_STAFF_ONLY', True)
LOGGER = getLogger('bambu_blog')

def clean_body(body):
    site = get_current_site()
    html = PyQuery('<body>' + body + '</body>')

    for p in html('p'):
        p = PyQuery(p)
        p.replaceWith('\n\n%s\n\n' % p.html())

    html('.alignright').addClass('pull-right').removeClass('alignright')
    html('.alignleft').addClass('pull-left').removeClass('alignleft')
    html('[style="float: left;"]').removeAttr('style').addClass('alignleft')
    html('[style="float: right;"]').removeAttr('style').addClass('alignright')

    while '\n\n\n' in body:
        body = body.replace('\n\n\n', '\n\n')

    while '\r\r\r' in body:
        body = body.replace('\r\r\r', '\r\r')

    body = html.html()
    body = body.replace('<br />', '  \n')
    body = body.replace('<br/>', '  \n')
    body = body.replace('<br>', '  \n')
    body = body.replace('\r\n', '\n')
    body = body.replace('\n\r', '\n')

    while body.find('\n\n\n') > -1:
        body = body.replace('\n\n\n', '\n\n')

    while body.startswith('\n'):
        body = body[1:]

    while body.endswith('\n'):
        body = body[:-1]

    while body.startswith('\r'):
        body = body[1:]

    while body.endswith('\r'):
        body = body[:-1]

    while body.startswith('\t'):
        body = body[1:]

    return body

def get_attachments(body):
    if body:
        html = PyQuery('<body>' + body + '</body>')

        for a in html('[src], [href]'):
            a = PyQuery(a)
            if a.attr('href'):
                url = a.attr('href')
            else:
                url = a.attr('src')

            try:
                upload = PostUpload.objects.get(url__iexact = url)
            except PostUpload.DoesNotExist:
                continue

            yield upload

def auth(username, password, *perms):
    user = authenticate(username = username, password = password)
    if not user:
        raise XMLRPCException('Login failed.', code = 401)

    if not user.is_active:
        raise XMLRPCException('User is inactive.', code = 401)

    if STAFF_ONLY and not user.is_staff:
        raise XMLRPCException('User is not a member of staff.', code = 401)

    if any(perms) and not user.has_perms(perms):
        raise XMLRPCException('User does not have permission to perform this action.', code = 401)

    return user

@handler('metaWeblog.getPost')
def get_post(postid, username, password):
    """The ``metaWeblog.getPost`` handler"""

    user = auth(username, password)
    site = get_current_site()

    try:
        post = Post.objects.get(pk = postid, author = user)
    except Post.DoesNotExist:
        raise XMLRPCException('Post not found.', code = 404)

    ret = {
        'postid': post.pk,
        'title': post.title or u'',
        'description': post.body or u'',
        'link': 'http://%s%s' % (site.domain, post.get_absolute_url()),
        'userid': post.author.username,
        'dateCreated': post.date,
        'categories': [
            c for c in post.categories.exclude(name = '').exclude(name = None).values_list(
                'name', flat = True
            )
        ],
        'mt_keywords': ', '.join(
            [
                t for t in post.tags.exclude(name = '').exclude(name = None).values_list(
                    'name', flat = True
                )
            ]
        ),
        'wp_slug': post.slug,
        'post_status': post.published and 'publish' or 'draft'
    }

    enclosures = post.attachments.filter(
        mimetype__in = (
            'video/avi',
            'video/msvideo',
            'video/x-msvideo',
            'audio/mpeg3',
            'audio/x-mpeg-3',
            'audio/x-wav',
            'video/mp4',
            'video/x-m4v',
            'video/mpeg'
        )
    )

    if enclosures.exists():
        enclosure = enclosures[0]

        ret['enclosure'] = {
            'url': enclosure.file.url,
            'length': enclosure.file.size,
            'type': enclosure.mimetype
        }

    return ret

@handler('metaWeblog.getRecentPosts')
def get_posts(blogid, username, password, numberOfPosts = 10):
    """The ``metaWeblog.getRecentPosts`` handler"""
    
    user = auth(username, password)
    site = get_current_site()

    if numberOfPosts > 10:
        numberOfPosts = 10

    posts = []
    for post in Post.objects.filter(author = user)[:numberOfPosts]:
        posts.append(
            {
                'postid': post.pk,
                'title': post.title or u'',
                'description': post.body or u'',
                'link': 'http://%s%s' % (site.domain, post.get_absolute_url()),
                'userid': post.author.username,
                'dateCreated': post.date,
                'categories': [
                    n for n in post.categories.exclude(name = '').exclude(name = None).values_list(
                        'name', flat = True
                    )
                ],
                'mt_keywords': ', '.join(
                    [
                        t for t in post.tags.exclude(name = '').exclude(name = None).values_list(
                            'name', flat = True
                        )
                    ]
                ),
                'wp_slug': post.slug,
                'post_status': post.published and 'publish' or 'draft'
            }
        )

    return posts

@handler('metaWeblog.newPost')
@transaction.atomic
def new_post(blogid, username, password, content, publish):
    """The ``metaWeblog.newPost`` handler"""

    user = auth(username, password, 'blog.create_posts')
    uploads = get_attachments(content.get('description'))
    post = Post.objects.create(
        author = user,
        title = content.get('title'),
        body = clean_body(content.get('description', '')),
        date = content.get('dateCreated', now()),
        slug = content.get('wp_slug', slugify(content.get('title'))) or None,
        published = publish
    )

    for upload in uploads:
        upload.convert_to_attachment(post)

    for name in content.get('categories', []):
        slug = slugify(name)

        try:
            category = Category.objects.get(slug = slug)
        except Category.DoesNotExist:
            category = Category.objects.create(
                name = name,
                slug = slug
            )

        post.categories.add(category)

    for tag in [t.strip() for t in content.get('mt_keywords', '').split(', ')]:
        if tag:
            post.tags.add(tag)

    return str(post.pk)

@handler('metaWeblog.editPost')
@transaction.atomic
def edit_post(postid, username, password, content, publish):
    """The ``metaWeblog.editPost`` handler"""

    user = auth(username, password, 'blog.change_posts')
    uploads = get_attachments(content.get('description'))

    try:
        post = Post.objects.get(pk = postid)
    except Post.DoesNotExist:
        raise XMLRPCException('Post not found.', code = 404)

    if post.author.pk != user.pk:
        raise XMLRPCException('Only the authoe can edit this post.', code = 401)

    post.title = content.get('title', post.title)

    if 'description' in content:
        post.body = clean_body(content['description'] or u'')

    post.date = content.get('dateCreated', post.date)
    post.slug = content.get('wp_slug', post.slug)
    post.published = publish
    post.save()

    for upload in uploads:
        upload.convert_to_attachment(post)

    if content.has_key('categories'):
        slugs = []
        for name in content['categories']:
            slug = slugify(name)
            if not slug:
                continue

            try:
                category = Category.objects.get(slug = slug)
            except Category.DoesNotExist:
                category = Category.objects.create(
                    name = name,
                    slug = slug
                )

            post.categories.add(category)
            slugs.append(category.slug)

        for pk in post.categories.exclude(slug__in = slugs).values_list('pk', flat = True):
            post.categories.remove(pk)

    if content.get('mt_keywords'):
        added_tags = []
        for tag in [t.strip() for t in content['mt_keywords'].split(', ')]:
            if tag.startswith('"') and tag.endswith('"'):
                tag = tag[1:-1]
            elif tag.startswith("'") and tag.endswith("'"):
                tag = tag[1:-1]

            if tag:
                post.tags.add(tag)
                added_tags.append(tag)

        for pk in post.tags.exclude(name__in = added_tags).values_list('pk', flat = True):
            post.tags.remove(pk)

    return True

@handler('metaWeblog.newMediaObject')
@transaction.atomic
def upload_media(blogid, username, password, data):
    """The ``metaWeblog.newMediaObject`` handler"""

    if not data.has_key('bits'):
        raise XMLRPCException('No data supplied for attachment.')

    handle, filename = mkstemp(
        data.get('name') and path.splitext(data['name'])[-1]
    )

    write(handle, data['bits'].data)
    close(handle)

    upload = PostUpload.objects.create(
        file = File(open(filename, 'r')),
        mimetype = data.get('type')
    )

    upload.url = upload.file.url
    upload.save()

    return {
        'id': upload.pk,
        'file': upload.file.name,
        'url': upload.url,
        'type': upload.mimetype
    }

@handler('metaWeblog.deletePost')
@handler('blogger.deletePost')
@transaction.atomic
def delete_post(appkey, postid, username, password, publish):
    """The ``metaWeblog.deletePost`` handler (also conforms to ``blogger.deletePost``)"""

    user = auth(username, password, 'blog.delete_posts')

    try:
        post = Post.objects.get(pk = postid)
    except Post.DoesNotExist:
        raise XMLRPCException('Post not found.', code = 404)

    if post.author.pk != user.pk:
        raise XMLRPCException('Only the authoe can delete this post.', code = 401)

    post.delete()
    return True

@handler('metaWeblog.getCategories')
def get_categories(blogid, username, password):
    """The ``metaWeblog.getCategories`` handler"""

    user = auth(username, password)
    site = get_current_site()

    return [
        {
            'categoryId': category.pk,
            'parentId': 0,
            'categoryName': category.name,
            'categoryDescription': u'',
            'description': category.name,
            'htmlUrl': 'http://%s%s' % (site.domain,
                reverse('blog_posts_by_category', args = [category.slug])
            ),
            'rssUrl': 'http://%s%s' % (site.domain,
                reverse('blog_posts_by_category_feed', args = [category.slug])
            )
        } for category in Category.objects.all()
    ]

@handler('metaWeblog.getUsersBlogs')
@handler('blogger.getUsersBlogs')
def get_blogs(appkey, username, password):
    """The ``metaWeblog.getUsersBlogs`` handler (also conforms to ``blogger.getUsersBlogs``)"""

    user = auth(username, password)
    site = get_current_site()

    return [
        {
            'blogid': 1,
            'url': 'http://%s%s' % (site.domain, reverse('blog_posts')),
            'blogName': site.name,
            'isAdmin': user.is_staff,
            'xmlrpc': 'http://%s%s' % (site.domain, reverse('xmlrpc_server'))
        }
    ]

@handler('blogger.getUserInfo')
def get_blogs(appkey, username, password):
    """The ``blogger.getUserInfo`` handler"""
    user = auth(username, password)
    site = get_current_site()

    return {
        'userid': user.pk,
        'nickname': user.username,
        'firstname': user.first_name,
        'lastname': user.last_name,
        'url': 'http://%s%s' % (site.domain,
            reverse('blog_posts_by_author', args = [user.username])
        )
    }



from bambu_blog.helpers import view_filter, title_parts, get_comments_form
from bambu_blog.models import Post, Category
from datetime import datetime
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.db import transaction
from django.db.models import Count, F
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.utils.timezone import get_current_timezone, now as rightnow
from django.views.decorators.http import require_POST
from taggit.models import Tag

POSTS_PER_PAGE = getattr(settings, 'BLOG_POSTS_PER_PAGE', 10)
THUMBNAIL_WIDTH = getattr(settings, 'BLOG_THUMBNAIL_WIDTH',
    getattr(settings, 'OEMBED_WIDTH', 640)
)

COMMENTS_FORM_CLASS = get_comments_form()

def _context(request):
    def categories():
        return Category.objects.filter(
            posts__pk__in = Post.objects.live().values_list('pk', flat = True)
        ).annotate(
            post_count = Count('posts')
        )

    def dates():
        return Post.objects.live().dates('date', 'month').reverse()

    return {
        'categories': categories,
        'dates': dates,
        'menu_selection': 'blog',
        'THUMBNAIL_WIDTH': unicode(THUMBNAIL_WIDTH)
    }

def posts(request, **kwargs):
    templates = ['blog/posts.html']
    context = _context(request)
    breadcrumb_trail = []

    if 'year' in kwargs:
        if 'month' in kwargs:
            if 'day' in kwargs:
                date = datetime(
                    int(kwargs['year']),
                    int(kwargs['month']),
                    int(kwargs['day'])
                ).replace(
                    tzinfo = get_current_timezone()
                )

                context['day'] = date.strftime('%B %d, %Y')

                breadcrumb_trail = (
                    ('../../../', u'Blog'),
                    ('../../', date.strftime('%Y')),
                    ('../', date.strftime('%B')),
                    ('', date.strftime('%d'))
                )

                templates.insert(0, 'blog/posts-day.html')
            else:
                date = datetime(
                    int(kwargs['year']),
                    int(kwargs['month']),
                    1
                ).replace(
                    tzinfo = get_current_timezone()
                )

                context['month'] = date.strftime('%B %Y')

                breadcrumb_trail = (
                    ('../../', u'Blog'),
                    ('../', date.strftime('%Y')),
                    ('', date.strftime('%B'))
                )

                templates.insert(0, 'blog/posts-month.html')
        else:
            context['year'] = kwargs['year']
            templates.insert(0, 'blog/posts-year.html')

            breadcrumb_trail = (
                ('../', u'Blog'),
                ('', int(kwargs['year'])),
            )
    else:
        breadcrumb_trail = (
            ('', u'Blog'),
        )

    if 'category' in kwargs:
        category = get_object_or_404(Category, slug = kwargs['category'])
        context['category'] = category
        templates.insert(0, 'blog/posts-category.html')
        breadcrumb_trail = (
            ('../', u'Blog'),
            ('', category.name)
        )
    elif 'tag' in kwargs:
        tag = get_object_or_404(Tag, slug = kwargs['tag'])
        context['tag'] = tag
        templates.insert(0, 'blog/posts-tag.html')
        breadcrumb_trail = (
            ('../', u'Blog'),
            ('', tag.name)
        )
    elif 'username' in kwargs:
        author = get_object_or_404(User, username = kwargs['username'])
        context['author'] = author
        templates.insert(0, 'blog/posts-author.html')
        breadcrumb_trail = (
            ('../', u'Blog'),
            ('', author.get_full_name() or author.username)
        )

    posts = view_filter(**kwargs).prefetch_related(
        'tagged_items__tag'
    ).with_featured_attachments().with_comment_counts()
    paginator = Paginator(posts, POSTS_PER_PAGE)
    page = request.GET.get('page')

    try:
        posts = paginator.page(page)
    except PageNotAnInteger:
        posts = paginator.page(1)
    except EmptyPage:
        posts = paginator.page(paginator.num_pages)

    context['page'] = posts
    context['breadcrumb_trail'] = breadcrumb_trail
    context['title_parts'] = title_parts(**kwargs)
    context['body_classes'] = ['page-%d' % posts.number]

    if any(posts.object_list) and getattr(posts.object_list[0], 'featured_attachment_file', None):
        context['body_classes'].append('first-post-has-featured-attachment')

    return TemplateResponse(
        request,
        templates,
        context
    )

def post(request, year, month, day, slug):
    preview = False
    kwargs = {
        'date__year': int(year),
        'date__month': int(month),
        'date__day': int(day),
        'slug': slug
    }

    now = rightnow()
    if not request.user.is_staff:
        kwargs['date__lte'] = now
        kwargs['published'] = True

    try:
        post = Post.objects.select_related().with_featured_attachments().with_comment_counts().get(**kwargs)
    except:
        raise Http404('No Post matches the given query.')

    if not post.published or post.date > now:
        preview = True

    context = _context(request)
    context['post'] = post
    context['day'] = post.date.strftime('%B %d, %Y')
    context['breadcrumb_trail'] = (
        ('../../../../', u'Blog'),
        ('../../../', post.date.strftime('%Y')),
        ('../../', post.date.strftime('%B')),
        ('../', post.date.strftime('%d')),
        ('', unicode(post))
    )

    context['title_parts'] = (unicode(post), u'Blog')
    context['newer_posts'] = Post.objects.filter(date__gt = post.date).live()
    context['older_posts'] = Post.objects.filter(date__lt = post.date).live()

    if not request.GET.get('comment-sent'):
        initial = {}

        if request.user.is_authenticated():
            initial = {
                'name': request.user.get_full_name() or request.user.username,
                'email': request.user.email,
                'website': 'http://%s/' % get_current_site().domain
            }

        context['comment_form'] = COMMENTS_FORM_CLASS(initial = initial)

    context['body_classes'] = ['post-%s' % post.pk, 'post-%s' % post.slug]
    if preview:
        context['body_classes'].append('post-preview')
        context['preview'] = True

    if post.css:
        context['body_classes'].append('post-custom-css')

    if getattr(post, 'featured_attachment_file', None):
        context['body_classes'].append('post-featured-attachment')

    return TemplateResponse(
        request,
        'blog/post.html',
        context
    )

@require_POST
def post_comment(request, year, month, day, slug):
    try:
        post = Post.objects.live().select_related().get(
            date__year = int(year),
            date__month = int(month),
            date__day = int(day),
            slug = slug
        )
    except Post.DoesNotExist:
        raise Http404('Post not found.')

    form = COMMENTS_FORM_CLASS(request.POST)
    if request.POST.get('h0n3ytr4p'):
        return HttpResponse('')

    if form.is_valid():
        comment = form.save(commit = False)
        with transaction.atomic():
            if request.POST.get('content_type'):
                comment.content_type = ContentType.objects.get(
                    pk = int(request.POST['content_type'])
                )
            else:
                comment.content_type = ContentType.objects.get_for_model(post)

            if request.POST.get('object_id'):
                comment.object_id = comment.content_type.get_object_for_this_type(
                    pk = int(request.POST['object_id'])
                ).pk
            else:
                comment.object_id = post.pk

            comment.spam = comment.check_for_spam(request)
            comment.save()

            messages.add_message(
                request,
                messages.SUCCESS,
                u'Your comment has been submitted successfully.'
            )

            return HttpResponseRedirect(
                '%s?comment-sent=true' % post.get_absolute_url()
            )

    context = _context(request)
    context['post'] = post
    context['day'] = post.date.strftime('%B %d, %Y')
    context['newer_posts'] = Post.objects.filter(date__gt = post.date).live()
    context['older_posts'] = Post.objects.filter(date__lt = post.date).live()
    context['breadcrumb_trail'] = (
        ('../../../../../', u'Blog'),
        ('../../../../', post.date.strftime('%Y')),
        ('../../../', post.date.strftime('%B')),
        ('../../', post.date.strftime('%d')),
        ('../', unicode(post)),
        ('', u'Post comment')
    )

    context['title_parts'] = (unicode(post), u'Blog')
    context['comment_form'] = form
    context['comment_form_action'] = '.'
    context['body_classes'] = ['post-%s' % post.pk, 'post-%s' % post.slug]

    if post.css:
        context['body_classes'].append('post-custom-css')

    return TemplateResponse(
        request,
        'blog/post.html',
        context
    )



from bambu_attachments.helpers import upload_attachment_file
from bambu_attachments.models import Attachment
from bambu_blog import helpers, excerpt
from bambu_blog.querysets import *
from django.conf import settings
from django.contrib.contenttypes import fields as generic
from django.db import models, transaction
from django.template import Template, Context
from django.utils.timezone import now
from hashlib import md5
from mimetypes import guess_type
from taggit.managers import TaggableManager

if 'bambu_webhooks' in settings.INSTALLED_APPS:
    from bambu_webhooks import site, send

COMMENTS_MODEL = getattr(settings, 'BLOG_COMMENTS_MODEL', 'bambu_comments.Comment')

class Category(models.Model):
    """A category under which blog posts can be filed"""

    name = models.CharField(max_length = 100, db_index = True)
    """The category name"""

    slug = models.SlugField(max_length = 100, unique = True)
    """The category slug (unique)"""

    def __unicode__(self):
        return self.name

    @property
    def post_percent(self):
        """The percentage of total blog posts that are in this category"""
        count = float(getattr(self, 'post_count', self.posts.live().count()))
        all_count = float(Post.objects.live().count())

        return count / all_count * 100.0

    class Meta:
        ordering = ('name',)
        verbose_name_plural = 'categories'
        db_table = 'blog_category'
        app_label = 'bambu_blog'

class PostQuerySet(models.QuerySet):
    """
    A custom queryset that adds a few utility functions to the standard one
    """

    def live(self):
        """Returns only posts in the future that are marked as published"""
        return self.filter(
            date__lte = now(),
            published = True
        )

    def css(self, rendered = False):
        """Returns rendered CSS for all of the posts in the query"""
        if rendered:
            return '\n\n'.join(
                [
                    (post.render_css() or u'') for post in self.all()
                ]
            )
        else:
            return '\n\n'.join(
                [
                    (css or u'') for css in self.values_list('css', flat = True)
                ]
            )

    def with_featured_attachments(self, only = False):
        queryset = self.extra(
            select = {
                'featured_attachment_file': 'SELECT file FROM attachments_attachment AS a ' \
                'INNER JOIN django_content_type AS t ON t.id = a.content_type_id ' \
                'WHERE t.app_label = \'bambu_blog\' AND t.model = \'post\' ' \
                'AND a.object_id = blog_post.id AND a.featured'
            }
        )

        if only:
            queryset = queryset.filter(
                attachments__featured = True
            ).distinct('date', 'pk')

        return queryset

    def with_comment_counts(self):
        return self.extra(
            select = {
                'comment_count': 'SELECT COUNT(*) FROM comments_comment AS c ' \
                'INNER JOIN django_content_type AS t ON t.id = c.content_type_id ' \
                'WHERE t.app_label = \'bambu_blog\' AND t.model = \'post\' ' \
                'AND c.object_id = blog_post.id AND c.approved'
            }
        )

class Post(models.Model):
    """
    A blog post
    """

    author = models.ForeignKey('auth.User', related_name = 'blog_posts')
    """The ``auth.User`` that wrote the post"""

    title = models.CharField(max_length = 100, null = True, blank = True)
    """The title of the post (optional)"""

    slug = models.SlugField(max_length = 100, db_index = True)
    """The slug of the post (if no title is given, a slug is generated from the total count of blog
    posts"""

    date = models.DateTimeField(db_index = True)
    """The post date"""

    published = models.BooleanField(default = True)
    """Whether the post is published (``True`` by default)"""

    broadcast = models.BooleanField(default = False, editable = False)
    """Whether a webhook event was fired when the post was published"""

    body = models.TextField()
    """The body of the post. By default this is a plain text field within the admin, but you can install
    django-markitup or django-tinymce and the admin will automatically detect one of those apps.
    Regardless of textbox interface, the body is always run through PyQuery's HTML parser, in order to
    generate a viable, readable excerpt that ignores references to things like oEmbed resources and
    images, and flattens lists, headings, quotes etc"""

    excerpt = models.TextField(null = True, blank = True, editable = False)
    """A short, automatically-generated excerpt for the post"""

    css = models.TextField(null = True, blank = True)
    """Custom CSS for the post (great for making custom-designed pages for each post)"""

    tags = TaggableManager()
    """The tags assigned to the post (optional)"""

    categories = models.ManyToManyField(Category, related_name = 'posts', blank = True)
    """The categories assigned to the post (optional)"""

    attachments = generic.GenericRelation(Attachment)
    """The attachments (images and other media) attached to the post (optional)"""

    comments = generic.GenericRelation(COMMENTS_MODEL)
    """A generic link to a comment model, as defined in the ``BLOG_COMMENTS_MODEL`` setting"""

    objects = PostQuerySet.as_manager()

    @models.permalink
    def get_absolute_url(self):
        return (
            'blog_post', (
                str(self.date.year).zfill(4),
                str(self.date.month).zfill(2),
                str(self.date.day).zfill(2),
                self.slug
            )
        )

    def __unicode__(self):
        return self.title or u'(Untitled)'

    def next_post(self):
        """Returns the next blog post in the sequence (irrespective of category, tag or author)"""
        try:
            return Post.objects.live().filter(date__gt = self.date)[0]
        except:
            pass

    def previous_post(self):
        """Returns the previous blog post in the sequence (irrespective of category, tag or author)"""
        try:
            return Post.objects.live().filter(date__lt = self.date).latest()
        except:
            pass

    def render_css(self):
        """
        Runs CSS through Django's templating engine, allowing you to write CSS that references
        attachment URLs and the post ID without actually having to know that they are
        """

        template = Template(self.css)
        context = Context(
            {
                'attachments': self.attachments.all(),
                'slug': self.slug,
                'pk': self.pk,
                'id': self.pk
            }
        )

        return template.render(context)

    def featured_attachment(self):
        """Returns the first attachment for the post, that's marked as Featured"""
        if hasattr(self, 'featured_attachment_file'):
            return self.featured_attachment_file

        try:
            return self.attachments.filter(featured = True)[0]
        except IndexError:
            return None

    def save(self, *args, **kwargs):
        publish = False
        if self.pk:
            old = Post.objects.get(pk = self.pk)
            if self.published and not old.published:
                publish = True
        elif self.published:
            publish = True

        if not self.slug and not self.title:
            slug = str(
                Post.objects.filter(
                    date__year = self.date.year,
                    date__month = self.date.month,
                    date__day = self.date.day
                ).count() + 1
            )

            while Post.objects.filter(
                date__year = self.date.year,
                date__month = self.date.month,
                date__day = self.date.day,
                slug = slug
            ).exists():
                slug = str(int(slug) + 1)

            self.slug = slug

        if self.body:
            self.excerpt = excerpt(self.body)

        super(Post, self).save(*args, **kwargs)
        if publish and self.date <= now():
            self.publish()

    def publish(self):
        if 'bambu_webhooks' in settings.INSTALLED_APPS:
            send('post_published', self.author,
                {
                    'id': self.pk,
                    'title': self.title,
                    'slug': self.slug,
                    'date': self.date.strftime('%Y-%m-%d %H:%M:%S'),
                    'body': self.body,
                    'tags': [t for t in self.tags.values_list('slug', flat = True)],
                    'categories': [c for c in self.categories.values_list('slug', flat = True)],
                    'attachments': [
                        a.file.url for a in self.attachments.all()
                    ]
                },
                md5('blogpost:%d' % self.pk).hexdigest()
            )

        self.broadcast = True

    class Meta:
        ordering = ('-date',)
        get_latest_by = 'date'
        db_table = 'blog_post'
        app_label = 'bambu_blog'

class PostUpload(models.Model):
    """
    A temporary holding place for media uploaded via XML-RPC
    """

    file = models.FileField(max_length = 255, upload_to = upload_attachment_file)
    url = models.CharField(max_length = 255, db_index = True)
    size = models.PositiveIntegerField(editable = False)
    mimetype = models.CharField(max_length = 50, editable = False, db_index = True)

    def __unicode__(self):
        return self.title

    def convert_to_attachment(self, post):
        """Converts the temporary uploaded file into an attachment"""
        with transaction.atomic():
            attachment = post.attachments.create(
                file = self.file,
                size = self.size,
                mimetype = self.file.size
            )

            self.delete()

        return attachment

    def save(self, *args, **kwargs):
        if self.file and not self.mimetype:
            self.mimetype, encoding = guess_type(self.file.name)

        if not self.size:
            self.size = self.file.size

        self.url = self.file.url
        super(PostUpload, self).save(*args, **kwargs)

    class Meta:
        db_table = 'blog_post_upload'
        app_label = 'bambu_blog'

if 'bambu_webhooks' in settings.INSTALLED_APPS:
    site.register('post_published',
        description = 'Fired when a post is published',
        staff_only = True
    )



#!/usr/bin/env python
from setuptools import setup
from os import path

setup(
    name = 'bambu-blog',
    version = '3.2.2',
    description = 'A simple set of models for a basic blog, with some tools for custom-designed blog post writing',
    author = 'Steadman',
    author_email = 'mark@steadman.io',
    url = 'https://github.com/iamsteadman/bambu-blog',
    long_description = open(path.join(path.dirname(__file__), 'README')).read(),
    install_requires = [
        'Django>=1.8',
        'pyquery',
        'html2text',
        'django-taggit',
        'sorl-thumbnail',
        'bambu-markup>=3.0',
        'bambu-oembed>=3.0',
        'bambu-attachments>=3.0',
        'bambu-xmlrpc>=3.0'
    ],
    packages = [
        'bambu_blog',
        'bambu_blog.management',
        'bambu_blog.management.commands',
        'bambu_blog.migrations',
        'bambu_blog.templatetags'
    ],
    package_data = {
        'bambu_blog': [
            'templates/blog/*.html',
            'templates/search/indexes/bambu_blog/*.txt',
            'static/blog/*.js'
        ]
    },
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django'
    ]
)
